Package: alevinQC
Type: Package
Title: Generate QC Reports For Alevin Output
Version: 1.27.0
Date: 2025-07-17
Authors@R: c(person("Charlotte", "Soneson", role = c("aut", "cre"), 
                    email = "charlottesoneson@gmail.com", 
                    comment = c(ORCID = "0000-0003-3833-2169")),
    person("Avi", "Srivastava", role = "aut"),
    person("Rob", "Patro", role = "aut"),
    person("Dongze", "He", role = "aut"))
Description: Generate QC reports summarizing the output from an alevin, 
    alevin-fry, or simpleaf run. 
    Reports can be generated as html or pdf files, or as shiny applications.
Encoding: UTF-8
SystemRequirements: C++11
Depends: R (>= 4.0)
Imports: rmarkdown (>= 2.5), tools, methods, ggplot2 (>= 3.4.0),
        GGally, dplyr, rjson, shiny, shinydashboard, DT, stats, utils,
        tximport (>= 1.17.4), cowplot, rlang, Rcpp
RoxygenNote: 7.3.2
Suggests: knitr, BiocStyle, testthat (>= 3.0.0), BiocManager
VignetteBuilder: knitr
biocViews: QualityControl, SingleCell
URL: https://github.com/csoneson/alevinQC
BugReports: https://github.com/csoneson/alevinQC/issues
License: MIT + file LICENSE
Config/testthat/edition: 3
LinkingTo: Rcpp
git_url: https://git.bioconductor.org/packages/alevinQC
git_branch: devel
git_last_commit: 3024244
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: yes
Packaged: 2025-10-30 20:01:09 UTC; biocbuild
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>),
  Avi Srivastava [aut],
  Rob Patro [aut],
  Dongze He [aut]
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>
