# Generated by roxygen2: do not edit by hand

export(alevinFryQCReport)
export(alevinFryQCShiny)
export(alevinQCReport)
export(alevinQCShiny)
export(checkAlevinFryInputFiles)
export(checkAlevinInputFiles)
export(plotAlevinBarcodeCollapse)
export(plotAlevinHistogram)
export(plotAlevinKneeNbrGenes)
export(plotAlevinKneeRaw)
export(plotAlevinQuant)
export(plotAlevinQuantPairs)
export(readAlevinFryQC)
export(readAlevinQC)
export(simpleafQCReport)
export(simpleafQCShiny)
import(dplyr)
import(rlang)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,renderDataTable)
importFrom(GGally,ggally_cor)
importFrom(GGally,ggally_densityDiag)
importFrom(GGally,ggally_points)
importFrom(GGally,ggpairs)
importFrom(Rcpp,sourceCpp)
importFrom(cowplot,get_legend)
importFrom(cowplot,plot_grid)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rug)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(methods,is)
importFrom(rjson,fromJSON)
importFrom(rlang,.data)
importFrom(rmarkdown,pandoc_available)
importFrom(rmarkdown,pandoc_exec)
importFrom(rmarkdown,render)
importFrom(shiny,actionButton)
importFrom(shiny,fluidRow)
importFrom(shiny,icon)
importFrom(shiny,observeEvent)
importFrom(shiny,plotOutput)
importFrom(shiny,renderPlot)
importFrom(shiny,shinyApp)
importFrom(shiny,stopApp)
importFrom(shinydashboard,box)
importFrom(shinydashboard,dashboardHeader)
importFrom(shinydashboard,dashboardPage)
importFrom(shinydashboard,dashboardSidebar)
importFrom(shinydashboard,dropdownMenu)
importFrom(shinydashboard,dropdownMenuOutput)
importFrom(shinydashboard,notificationItem)
importFrom(shinydashboard,renderMenu)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(tximport,tximport)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
useDynLib(alevinQC, .registration = TRUE)
