# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_get_permit_freq_info <- function(fn) {
    .Call(`_alevinQC_cpp_get_permit_freq_info`, fn)
}

